<?php
namespace Controller;

use \Service\Subrelease as SubreleaseService;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\SocketClient;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row as RequestRow;
use Zeedhi\Framework\DTO\Request as Request;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;

class Subrelease extends \Generated\Controller\Subrelease {
    protected $subreleaseService;

    /**
     * @param Manager $dataSourceManager
     * @param SubreleaseService $subreleaseService
     */
    public function __construct(Manager $dataSourceManager, SubreleaseService $subreleaseService) {
        $this->subreleaseService = $subreleaseService;
        parent::__construct($dataSourceManager);
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveSubrelease(RequestDataset $request, Response $response) {
        try {
            $subreleaseRows         = $request->getDataSet()->getRows();
            $subrelease             = $subreleaseRows[0];
            $id                     = $subrelease['ID'];
            $name                   = $subrelease['NAME'];
            $active                 = $subrelease['IS_ACTIVE'];
            $releaseId              = $subrelease['RELEASE_ID'];
            $userId                 = empty($subrelease['ZH_USER']) ? $subrelease['USER_ID'] : $subrelease['ZH_USER'];
            $productServerId        = $subrelease['PRODUCT_SERVER_ID'];
            $status                 = $subrelease['STATUS'];
            $version                = $subrelease['VERSION'];
            $revision               = $subrelease['REVISION'];
            $build                  = $subrelease['BUILD'];
            $description            = $subrelease['DESCRIPTION'];
            $projectId              = isset($subrelease['PROJECT_ID']) ? $subrelease['PROJECT_ID'] : null;
            $branchToDeploy         = isset($subrelease['BRANCH_TO_DEPLOY']) ? $subrelease['BRANCH_TO_DEPLOY'] : null;
            $pathToTest             = isset($subrelease['PATH_TO_TEST']) ? $subrelease['PATH_TO_TEST'] : null;
            $urlToTest              = isset($subrelease['URL_TO_TEST']) ? $subrelease['URL_TO_TEST'] : null;
            $releaseProductServerId = isset($subrelease['RELEASE_PRODUCT_SERVER_ID']) ? $subrelease['RELEASE_PRODUCT_SERVER_ID'] : null;

            $id = $this->subreleaseService->saveSubrelease($id, $name, $active, $releaseId, $userId, $productServerId,
                $status, $version, $revision, $build, $description, $projectId, $branchToDeploy, $pathToTest, $urlToTest, $releaseProductServerId);
            $response->addDataSet(new DataSet('subrelease', array($id)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */
    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $request     = $this->buildDataSetIfRow($request);
        $dataSet     = $request->getDataSet();
        $showMessage = $this->verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    /**
     * @param $dataSet
     * @return bool
     */
    private function verifyShowMessage($dataSet) {
        $showMessage = true;

        foreach ($dataSet->getRows() as $row) {
            if(isset($row['isMessage']) && $row['isMessage'] === false) {
                $showMessage = false;
                break;
            }
        }

        return $showMessage;
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findSubrelease(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        try {
            $registers = $this->subreleaseService->findSubrelease($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('subrelease', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function checkPermissionSub(RequestFilter $request, Response $response){
        try {
            $conditions    = $request->getFilterCriteria()->getConditions();
            $subReleaseId  = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
            $result        = $this->subreleaseService->checkPermissionSub($subReleaseId);
            $response->addDataSet(new DataSet('checkPermissionSub', [$result]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findTaskNotIsCompletedOfIssues(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();

        try {
            $projectId = Utilities::getValueInFilterDataSourceByName($conditions,'PROJECT_ID');
            $registers = $this->subreleaseService->findTaskNotIsCompletedOfIssues($projectId);
            Utilities::setDataSetDefault($request,$response,$registers);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function generateTestEnvironmentScript(RequestDataset $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $productRepositories = $row['productRepositories'];
            $currentRow = $row['currentRow'];
            $branchToHomologate = $row['branchToHomologate'];
            $pathToTest = $row['pathToTest'];
            $productFiles = $row['productFiles'];
            $scriptBeforeProductFiles = $row['scriptBeforeProductFiles'];
            $scriptAfterProductFiles = $row['scriptAfterProductFiles'];
            $repositoriesWithNewPath = $row['repositoriesWithNewPath'];
            $subReleaseName = $row['subReleaseName'];
            $configurationFiles = $row['configurationFiles'];
            $this->subreleaseService->generateTestEnvironmentScript($productRepositories, $currentRow, $branchToHomologate,
                                                                    $pathToTest, $productFiles, $scriptBeforeProductFiles,
                                                                    $scriptAfterProductFiles, $repositoriesWithNewPath,
                                                                    $subReleaseName, $configurationFiles);
            Utilities::setDataSetDefault($request, $response, []);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function testEnvironmentScriptDownload(Request $request, Response $response)
    {
        try{
            $this->subreleaseService->testEnvironmentScriptDownload();
        } catch (\Exception $e){
            $response->addMessage(new DTO\Response\Message($e->getMessage()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findServersBySubReleaseAndType(RequestFilter $request, Response $response){
        $conditions   = $request->getFilterCriteria()->getConditions();
        $subReleaseId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        $serverType   = empty($conditions[1]) ? '0' : $conditions[1]['value'];
        try {
            $registers = $this->subreleaseService->findServersBySubReleaseAndType($subReleaseId, $serverType);
            $response->addDataSet(new DataSet('serversBySubReleaseAndType', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findTestOrPackageServersByProduct(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $productId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
        $filter = Utilities::getFilterTableColumn($conditions, 'SERVER_NAME');

        try {
            $registers = $this->subreleaseService->findTestOrPackageServersByProduct($productId, $filter);
            $response->addDataSet(new DataSet('serversByProductAndType', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findProductFilesBySubRelease(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $subReleaseId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        try {
            $registers = $this->subreleaseService->findProductFilesBySubRelease($subReleaseId);
            $response->addDataSet(new DataSet('registersSubReleaseTest', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findSelectedProductFilesByProduct(RequestFilter $request, Response $response){
        $conditions   = $request->getFilterCriteria()->getConditions();
        $page         = $request->getFilterCriteria()->getPage();
        $pageSize     = $request->getFilterCriteria()->getPageSize();
        $productId    = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT', true);
        $subreleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE', true);
        $typeEvent    = Utilities::getValueInFilterDataSourceByName($conditions, 'TYPE_EVENT', true);
        $filter       = Utilities::createQueryFiltered($conditions);
        try {
            $selectedProductFiles = $this->subreleaseService->findSelectedProductFilesByProduct($productId, $subreleaseId, $typeEvent, $filter,$page, $pageSize);
            $response->addDataSet(new DataSet('selectedProductFilesByProduct', $selectedProductFiles));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function removeSubreleaseProductFiles(RequestDataset $request, Response $response) {
        try {
            $subreleaseProductFiles = $request->getDataSet()->getRows();
            foreach($subreleaseProductFiles as $productFile) {
                if($productFile['ID']) {
                    $this->subreleaseService->removeSubreleaseProductFiles($productFile['ID']);
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }


    public function validateSubReleaseServerMnemonic(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUB_RELEASE');
            $typeEvent = Utilities::getValueInFilterDataSourceByName($conditions, 'TYPE_EVENT');
            $productFiles = $this->subreleaseService->validateSubReleaseServerMnemonic($subReleaseId, $typeEvent);
            $response->addDataSet(new DataSet('validateSubReleaseServerMnemonic', $productFiles));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveSubreleaseProductFiles(RequestDataset $request, Response $response) {
        try {
            $subreleaseProductFiles = $request->getDataSet()->getRows();
            foreach($subreleaseProductFiles as $productFile) {
                if(!$productFile['ID']) {
                    $this->subreleaseService->saveSubreleaseProductFiles($productFile['SUBRELEASE_ID'], $productFile['PRODUCT_FILES_ID']);
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function removePathSubRelease(RequestFilter $request, Response $response) {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $subReleaseId = Utilities::getValueInFilterDataSourceByName($filters,'SUBRELEASE_ID');
            $this->subreleaseService->removePathSubReleaseForController($subReleaseId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveProductFileRequiredOnce(RequestDataset $request, Response $response) {
        try {
            $subreleaseProductFiles = $request->getDataSet()->getRows();
            $productFile = $subreleaseProductFiles[0];
            $this->subreleaseService->saveProductFileRequiredOnce($productFile['SUBRELEASE_ID'], $productFile['PRODUCT_FILES_ID']);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findTasksNotConcluded(RequestFilter $request, Response $response){
        $conditions   = $request->getFilterCriteria()->getConditions();
        $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
        try {
            $taskNames = $this->subreleaseService->findTasksNotConcluded($subReleaseId);
            $response->addDataSet(new DataSet('tasksNotConcluded', $taskNames));
            $taskNames = $this->subreleaseService->validateFinishSubRelease($subReleaseId);
            $response->addDataSet(new DataSet('tasksGeneratedTest', $taskNames));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findTaskBranchesSubrelaseProject(RequestFilter $request, Response $response){
        $conditions   = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');
        try {
            $branchTasks = $this->subreleaseService->findTaskBranchesByProject($projectId);
            $response->addDataSet(new DataSet('branchTasks', $branchTasks));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findTasksNotTestGeneratedByBranches(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $branches  = Utilities::getValueInFilterDataSourceByName($conditions, 'BRANCHES');
        $projectId = Utilities::getValueInFilterDataSourceByName($conditions, 'PROJECT');
        try {
            $tasks = $this->subreleaseService->findTasksNotTestGeneratedByBranches($branches, $projectId);
            $response->addDataSet(new DataSet('tasksNotTestGenerated', $tasks));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findConflictTaskGeneratedByBranch(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $branch  = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $tasks = $this->subreleaseService->findConflictTaskGeneratedByBranch($branch);
            $response->addDataSet(new DataSet('conflictTaskGenerated', $tasks));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     * @throws \Exception
     */
    public function updateStatusSubReleaseAndVersion(RequestDataset $request, Response $response) {
        try {
            $subreleaseRows = $request->getDataSet()->getRows();
            $subrelease = $subreleaseRows[0];
            $version = $subrelease['VERSION'];
            $revision = $subrelease['REVISION'];
            $build = $subrelease['BUILD'];
            $subReleaseId = $subrelease['ID'];
            $status = $subrelease['STATUS'];
            $isProjectAssociation = $subrelease['IS_PROJECT_ASSOCIATION'];
            $packages = $subrelease['PACKAGES'];
            $modulesConfiguration = $subrelease['MODULES_CONFIGURATION'];

            $this->subreleaseService->homologateSubRelease($subReleaseId, $version, $revision, $build, $status,
                $isProjectAssociation, $packages, $modulesConfiguration);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     * @throws \Exception
     */
    public function createLogFile(RequestDataset $request, Response $response) {
        try {
            $logRows = $request->getDataSet()->getRows();
            $log = $logRows[0];
            $logFileName = $log['FILE_NAME'];
            $logContents = $log['FILE_CONTENTS'];

            $this->subreleaseService->createLogFile($logFileName, $logContents);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function getLogFileContents(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $logFileName  = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $logFileContents = $this->subreleaseService->getLogFileContents($logFileName);
            $response->addDataSet(new DataSet('logFileContents', $logFileContents));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findIssuesPartialCompletedBySubrelease(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $subreleaseId = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $issues = $this->subreleaseService->findIssuesPartialCompletedBySubreleaseId($subreleaseId);
            $response->addDataSet(new DataSet('issuesPartialCompleted', $issues));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findRelatedIssuesNotHomologatedBySubrelease(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $subreleaseId = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $issues = $this->subreleaseService->findRelatedIssuesNotHomologatedBySubrelease($subreleaseId);
            $response->addDataSet(new DataSet('relatedIssuesNotHomologated', $issues));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestRow $request
     * @param Response $response
     */
    public function removeSubrelease(RequestRow $request, Response $response) {
        try {
            $subrelease   = $request->getRow();
            $subreleaseId = $subrelease['ID'];

            $this->subreleaseService->removeSubrelease($subreleaseId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     * @throws \Exception
     */
    public function updateTasksUrlToTest(RequestDataset $request, Response $response) {
        try {
            $taskRows = $request->getDataSet()->getRows();
            $tasks = $taskRows;

            $this->subreleaseService->updateTasksUrlToTest($tasks);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findTaskByBranch(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $branchName = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $task = $this->subreleaseService->findTaskByBranch($branchName);
            $response->addDataSet(new DataSet('taskByBranch', $task));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findDynamicProjects(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $conditions = empty($conditions[0]) ? [] : $conditions[0];
        $projectId = empty($conditions) ? '' : $conditions['value'];
        $fieldName = empty($conditions) ? '' : $conditions['columnName'];
        try {
            $task = $this->subreleaseService->findDynamicProjects($projectId, $fieldName);
            $response->addDataSet(new DataSet('testProjects', $task));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findTestTasks(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $projectId = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $task = $this->subreleaseService->findTestTasks($projectId);
            $response->addDataSet(new DataSet('testTasks', $task));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     * @throws \Exception
     */
    public function saveTestProjectRelationship(RequestDataset $request, Response $response) {
        try {
            $projectRow = $request->getDataSet()->getRows();
            $projectRow = $projectRow[0];
            $projectId  = $projectRow['PROJECT'];
            $projectRelationshipId  = $projectRow['PROJECT_RELATIONSHIP'];
            $this->subreleaseService->saveTestProjectRelationship($projectId, $projectRelationshipId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findBranchesAllowedExclusion(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $subreleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
        try {
            $task = $this->subreleaseService->findBranchesAllowedExclusion($subreleaseId);
            $response->addDataSet(new DataSet('branches', $task));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function validateIssueHomologation(RequestFilter $request, Response $response) {
        $conditions   = $request->getFilterCriteria()->getConditions();
        $subreleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
        $releaseId    = Utilities::getValueInFilterDataSourceByName($conditions, 'RELEASE_ID');
        try {
            $validation = $this->subreleaseService->validateIssueHomologation($subreleaseId, $releaseId);
            $response->addDataSet(new DataSet('validateIssueHomologation', $validation));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function validateSubreleasePreviousVersionsNotFinalized(RequestFilter $request, Response $response) {
        $conditions   = $request->getFilterCriteria()->getConditions();
        $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
        try {
            $validation = $this->subreleaseService->validateSubreleasePreviousVersionsNotFinalized($subReleaseId);
            $response->addDataSet(new DataSet('validateSubreleasePreviousVersionsNotFinalized', $validation));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function checkCreateBuildToCurrentVersion(RequestFilter $request, Response $response) {
        $conditions   = $request->getFilterCriteria()->getConditions();
        $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
        try {
            $validation = $this->subreleaseService->checkCreateBuildToCurrentVersion($subReleaseId);
            $response->addDataSet(new DataSet('checkCreateBuildToCurrentVersion', $validation));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function checkIfProductValuesAreFullfilled(RequestFilter $request, Response $response) {
        try {
            $conditions   = $request->getFilterCriteria()->getConditions()[0];
            $subreleaseId = $conditions['value'];

            $result = $this->subreleaseService->checkIfProductValuesAreFullfilled($subreleaseId);
            $response->addDataSet(new DataSet('checkIfProductValuesAreFullfilled', $result));

        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findResponsesOfIssuesAndTasks(RequestFilter $request, Response $response)
    {
        try {
            $conditions   = $request->getFilterCriteria()->getConditions();
            $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUB_RELEASE_ID', true);
            $useGitAuto = Utilities::getValueInFilterDataSourceByName($conditions, 'USEGITAUTOMATICALLY', true);
            $issuesResponses = $this->subreleaseService->findResponsesOfIssuesAndTasks($subReleaseId, $conditions, $useGitAuto);
            $response->addDataSet(new DataSet('findResponsesOfIssuesAndTasks', $issuesResponses));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findTaskResponsesRelatedSameIssue(RequestFilter $request, Response $response)
    {
        try {
            $conditions   = $request->getFilterCriteria()->getConditions();
            $issueId = Utilities::getValueInFilterDataSourceByName($conditions, 'ISSUE_ID', true);
            $taskResponses = $this->subreleaseService->findTaskResponsesRelatedSameIssue($issueId);
            $response->addDataSet(new DataSet('findTaskResponsesRelatedSameIssue', $taskResponses));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveResponsesOfIssuesAndTasks(RequestDataset $request, Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->subreleaseService->saveResponsesOfIssuesAndTasks($rows);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findProductInfo(RequestFilter $request, Response $response)
    {
        try {
            $conditions   = $request->getFilterCriteria()->getConditions();
            $productId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productInfo = $this->subreleaseService->findProductInfo($productId);
            $response->addDataSet(new DataSet('findProductInfo', $productInfo));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findMigrationInfo(RequestFilter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $migrationConfig = Utilities::getValueInFilterDataSourceByName($conditions, 'MIGRATION', true);
            $migrationInfo   = $this->subreleaseService->findMigrationInfo($migrationConfig);
            $response->addDataSet(new DataSet('findMigrationInfo', $migrationInfo));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     * @throws \Exception
     */
    public function saveMigrationScript(RequestDataset $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $deployId = empty($row['subReleaseId']) ? null : $row['subReleaseId'] ;
            $document = empty($row['DOCUMENT']) ? null : $row['DOCUMENT'];
            $type = empty($row['TYPE']) ? null : $row['TYPE'];
            $savedFiles = $this->subreleaseService->saveLog($document, $deployId, $type);
            $response->addDataSet(new DataSet('savedFiles', $savedFiles));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findDeployMigrations(RequestFilter $request, Response $response)
    {
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $subReleaseId    = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID', true);
            $attachments = $this->subreleaseService->findDeployMigrations($subReleaseId, $conditions);
            $response->addDataSet(new DataSet('findDeployMigrations', $attachments));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function downloadDeployMigration(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $attachmentId = $row['ID'];
            $this->subreleaseService->downloadDeployMigration($attachmentId);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function setPackageCreatedStatusInSubRelease(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $subReleaseId = $row['SUBRELEASE_ID'];
            $this->subreleaseService->setPackageCreatedStatusInSubRelease($subReleaseId);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode(), $ex->getTraceAsString()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function validateSubreleasesInTestOrPendingToFinish(RequestFilter $request, Response $response) {
        $conditions   = $request->getFilterCriteria()->getConditions();
        $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
        try {
            $validation = $this->subreleaseService->validateSubreleasesInTestOrPendingToFinish($subReleaseId);
            $response->addDataSet(new DataSet('validateSubreleasesInTestOrPendingToFinish', $validation));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function blockOtherSubreleaseTestTasks(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $subReleaseId = $row['SUBRELEASE_ID'];
            $urlToTest    = $row['URL_TO_TEST'];
            $this->subreleaseService->blockOtherSubreleaseTestTasks($subReleaseId, $urlToTest);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode(), $ex->getTraceAsString()));
        }
    }
}